@@ -50,21 +50,9 @@ namespace Xapian {
    
 ExpandDeciderFilterTerms::ExpandDeciderFilterTerms(TermIterator terms,
 						   TermIterator termsend)
-#ifndef __SUNPRO_CC
     : tset(terms, termsend)
 {
 }
-#else
-{
-    // I'd prefer using an initialiser list for this, but it seems
-    // that Solaris' CC doesn't like initialising a set with list
-    // iterators or Xapian::TermIterators.
-    while (terms != termsend) {
-        tset.insert(*terms);
-	++terms;
-    }
-}
-#endif
 
 int
 ExpandDeciderFilterTerms::operator()(const string &tname) const
@@ -146,19 +134,6 @@ RSet::get_description() const
     RETURN("RSet(" + internal->get_description() + ")");
 }
 
-string
-RSet::Internal::to_string() const
-{
-    string result = om_tostring(items.size());
-
-    set<Xapian::docid>::const_iterator i;
-    for (i = items.begin(); i != items.end(); ++i) {
-	result += ' ';
-	result += om_tostring(*i);
-    }
-    return result;
-}
-
 string
 RSet::Internal::get_description() const
 {
@@ -634,6 +609,12 @@ MSetIterator::get_weight() const
     return mset.internal->items[index].wt;
 }
 
+std::string
+MSetIterator::get_collapse_key() const
+{
+    return mset.internal->items[index].collapse_key;
+}
+
 Xapian::doccount
 MSetIterator::get_collapse_count() const
 {
