@@ -1,9 +1,9 @@
 /* omqueryinternal.cc: Internals of query interface
  *
- * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,20 +17,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
- * -----END-LICENCE-----
  */
 
 #include <config.h>
 #include "omdebug.h"
 #include "omqueryinternal.h"
 #include "utils.h"
-#include "netutils.h"
+#include "serialise.h"
 
 #include <xapian/error.h>
 #include <xapian/enquire.h>
 #include <xapian/termiterator.h>
+#include <xapian/version.h>
 #include "vectortermlist.h"
 
 #include <vector>
@@ -101,14 +101,14 @@ is_leaf(Xapian::Query::Internal::op_t op)
  *  The format is designed to be relatively easy
  *  to parse, as well as encodable in one line of text.
  *
- *  A single-term query becomes `[<encodedtname> @<termpos>#<wqf>'
+ *  A single-term query becomes `[<encodedtname>@<termpos>#<wqf>'
  *  where:
- *  	<wqf> is the decimal within query frequency (1 if omitted),
- *  	<termpos> is the decimal term position (index of term if omitted).
+ *	<wqf> is the decimal within query frequency (1 if omitted),
+ *	<termpos> is the decimal term position (index of term if omitted).
  *
  *  A compound query becomes `(<subqueries><op>', where:
- *  	<subqueries> is the list of subqueries
- *  	<op> is one of: &|%+-^
+ *	<subqueries> is the list of subqueries
+ *	<op> is one of: &|%+-^
  *  also ~N "N >F *N (N unsigned int; F floating point)
  * 
  *  If querylen != sum(wqf) we append `=len' (at present we always do this
@@ -118,12 +118,14 @@ is_leaf(Xapian::Query::Internal::op_t op)
 string
 Xapian::Query::Internal::serialise() const
 {
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
     Xapian::termpos curpos = 1;
     string result;
 
     if (op == Xapian::Query::Internal::OP_LEAF) {
-	result += "[" + encode_tname(tname);
-	result += ' ';
+	result += '[';
+	result += encode_length(tname.length());
+	result += tname;
        	if (term_pos != curpos) result += '@' + om_tostring(term_pos);
 	if (wqf != 1) result += '#' + om_tostring(wqf);
 	++curpos;
@@ -168,6 +170,9 @@ Xapian::Query::Internal::serialise() const
 	}
     }
     return result;
+#else
+    throw Xapian::InternalError("query serialisation not compiled in");
+#endif
 }
 
 string
@@ -243,10 +248,10 @@ Xapian::Query::Internal::accumulate_terms(
         // We're a leaf, so just return our term.
         terms.push_back(make_pair(tname, term_pos));
     } else {
-    	subquery_list::const_iterator end = subqs.end();
+	subquery_list::const_iterator end = subqs.end();
         // not a leaf, concatenate results from all subqueries
 	for (subquery_list::const_iterator i = subqs.begin(); i != end; ++i) {
- 	    (*i)->accumulate_terms(terms);
+	    (*i)->accumulate_terms(terms);
 	}
     }
 }
@@ -273,7 +278,7 @@ Xapian::Query::Internal::get_terms() const
     // remove adjacent duplicates, and return an iterator pointing
     // to just after the last unique element
     vector<pair<string, Xapian::termpos> >::iterator newlast =
-	    	unique(terms.begin(), terms.end());
+		unique(terms.begin(), terms.end());
     // and remove the rest...  (See Stroustrup 18.6.3)
     terms.erase(newlast, terms.end());
 
@@ -287,41 +292,45 @@ Xapian::Query::Internal::get_terms() const
 						   result.end()));
 }
 
-// Methods 
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
+// Methods.
 
 class QUnserial {
   private:
     const char *p;
+    const char *end;
     Xapian::termpos curpos;
- 
+
     Xapian::Query::Internal * readquery();
     Xapian::Query::Internal * readcompound();
-    
+
   public:
-    QUnserial(const char *p_) : p(p_), curpos(1) { }
+    QUnserial(const string & s) : p(s.c_str()), end(p + s.size()), curpos(1) { }
     Xapian::Query::Internal * decode();
 };
 
 Xapian::Query::Internal *
 QUnserial::decode() {
     DEBUGLINE(UNKNOWN, "QUnserial::decode(" << p << ")");
-    Xapian::Query::Internal * qint = readquery();
-    DEBUGLINE(UNKNOWN, "Remainder of query (should be none) is `" << p << "'");
-    Assert(*p == '\0');
-    return qint;
+    AutoPtr<Xapian::Query::Internal> qint(readquery());
+    if (p != end)
+        throw Xapian::InvalidArgumentError("Bad serialised query");
+    return qint.release();
 }
 
 Xapian::Query::Internal *
 QUnserial::readquery() {
+    if (p == end)
+	throw Xapian::InvalidArgumentError("Bad serialised query");
     switch (*p++) {
 	case '[': {
-	    const char *q = strchr(p, ' ');
-	    if (!q) q = p + strlen(p);
-	    string tname = decode_tname(string(p, q - p));
+	    size_t length = decode_length(&p, end);
+	    string tname(p, length);
+	    p += length;
 	    Xapian::termpos term_pos = curpos;
 	    Xapian::termcount wqf = 1;
-	    p = q;
-	    if (*p == ' ') ++p;
+	    if (p == end)
+		throw Xapian::InvalidArgumentError("Bad serialised query");
 	    if (*p == '@') {
 		char *tmp; // avoid compiler warning
 		term_pos = strtol(p + 1, &tmp, 10);
@@ -350,8 +359,10 @@ qint_from_vector(Xapian::Query::op op,
 {
     Xapian::Query::Internal * qint = new Xapian::Query::Internal(op, parameter);
     vector<Xapian::Query::Internal *>::const_iterator i;
-    for (i = vec.begin(); i != vec.end(); i++)
+    for (i = vec.begin(); i != vec.end(); i++) {
 	qint->add_subquery(**i);
+	delete *i;
+    }
     qint->end_construction();
     return qint;
 }
@@ -359,51 +370,60 @@ qint_from_vector(Xapian::Query::op op,
 Xapian::Query::Internal *
 QUnserial::readcompound() {
     vector<Xapian::Query::Internal *> subqs;
-    while (true) {
-	switch (*p++) {
-	    case '[':
-		--p;
-		subqs.push_back(readquery());
-		break;
-	    case '(': {
-		subqs.push_back(readcompound());
-		break;
+    try {
+        while (true) {
+	    if (p == end)
+	        throw Xapian::InvalidArgumentError("Bad serialised query");
+	    switch (*p++) {
+	        case '[':
+		    --p;
+		    subqs.push_back(readquery());
+		    break;
+	        case '(': {
+		    subqs.push_back(readcompound());
+		    break;
+	        }
+	        case '&':
+		    return qint_from_vector(Xapian::Query::OP_AND, subqs);
+	        case '|':
+		    return qint_from_vector(Xapian::Query::OP_OR, subqs);
+	        case '%':
+		    return qint_from_vector(Xapian::Query::OP_FILTER, subqs);
+	        case '^':
+		    return qint_from_vector(Xapian::Query::OP_XOR, subqs);
+	        case '+':
+		    return qint_from_vector(Xapian::Query::OP_AND_MAYBE, subqs);
+	        case '-':
+		    return qint_from_vector(Xapian::Query::OP_AND_NOT, subqs);
+	        case '~': {
+		    char *tmp; // avoid compiler warning
+		    Xapian::termcount window(strtol(p, &tmp, 10));
+		    p = tmp;
+		    return qint_from_vector(Xapian::Query::OP_NEAR, subqs, window);
+	        }
+	        case '"': {
+		    char *tmp; // avoid compiler warning
+		    Xapian::termcount window(strtol(p, &tmp, 10));
+		    p = tmp;
+		    return qint_from_vector(Xapian::Query::OP_PHRASE, subqs, window);
+	        }
+	        case '*': {
+		    char *tmp; // avoid compiler warning
+		    Xapian::termcount elite_set_size(strtol(p, &tmp, 10));
+		    p = tmp;
+		    return qint_from_vector(Xapian::Query::OP_ELITE_SET, subqs,
+					    elite_set_size);
+	        }
+	        default:
+		    DEBUGLINE(UNKNOWN, "Can't parse remainder `" << p - 1 << "'");
+		    throw Xapian::InvalidArgumentError("Invalid query string");
 	    }
-	    case '&':
-		return qint_from_vector(Xapian::Query::OP_AND, subqs);
-	    case '|':
-		return qint_from_vector(Xapian::Query::OP_OR, subqs);
-	    case '%':
-		return qint_from_vector(Xapian::Query::OP_FILTER, subqs);
-	    case '^':
-		return qint_from_vector(Xapian::Query::OP_XOR, subqs);
-	    case '+':
-		return qint_from_vector(Xapian::Query::OP_AND_MAYBE, subqs);
-	    case '-':
-		return qint_from_vector(Xapian::Query::OP_AND_NOT, subqs);
-	    case '~': {
-		char *tmp; // avoid compiler warning
-		Xapian::termcount window(strtol(p, &tmp, 10));
-		p = tmp;
-		return qint_from_vector(Xapian::Query::OP_NEAR, subqs, window);
-	    }
-	    case '"': {
-		char *tmp; // avoid compiler warning
-		Xapian::termcount window(strtol(p, &tmp, 10));
-		p = tmp;
-		return qint_from_vector(Xapian::Query::OP_PHRASE, subqs, window);
-	    }
-	    case '*': {
-		char *tmp; // avoid compiler warning
-		Xapian::termcount elite_set_size(strtol(p, &tmp, 10));
-		p = tmp;
-		return qint_from_vector(Xapian::Query::OP_ELITE_SET, subqs,
-					elite_set_size);
-	    }
-	    default:
-		DEBUGLINE(UNKNOWN, "Can't parse remainder `" << p - 1 << "'");
-		throw Xapian::InvalidArgumentError("Invalid query string");
-	}
+        }
+    } catch(...) {
+        vector<Xapian::Query::Internal *>::iterator i;
+        for (i = subqs.begin(); i != subqs.end(); i++)
+            delete *i;
+        throw;
     }
 }
 
@@ -411,11 +431,18 @@ Xapian::Query::Internal *
 Xapian::Query::Internal::unserialise(const string &s)
 {
     Assert(s.length() > 1);
-    QUnserial u(s.c_str());
+    QUnserial u(s);
     Xapian::Query::Internal * qint = u.decode();
     AssertEq(s, qint->serialise());
     return qint;
 }
+#else
+Xapian::Query::Internal *
+Xapian::Query::Internal::unserialise(const string &)
+{
+    throw Xapian::InternalError("query serialisation not compiled in");
+}
+#endif
 
 /** Swap the contents of this with another Xapian::Query::Internal,
  *  in a way which is guaranteed not to throw.  This is
