@@ -32,6 +32,7 @@
 #include <xapian/error.h>
 #include <xapian/valueiterator.h>
 
+#include "flint_modifiedpostlist.h"
 #include "flint_postlist.h"
 #include "flint_termlist.h"
 #include "flint_positionlist.h"
@@ -70,7 +71,7 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 			       unsigned int block_size)
 	: db_dir(flint_dir),
 	  readonly(action == XAPIAN_DB_READONLY),
-	  metafile(db_dir + "/iamflint"),
+	  version_file(db_dir),
 	  postlist_table(db_dir, readonly),
 	  positionlist_table(db_dir, readonly),
 	  termlist_table(db_dir, readonly),
@@ -157,7 +158,6 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 FlintDatabase::~FlintDatabase()
 {
     DEBUGCALL(DB, void, "~FlintDatabase", "");
-    // Only needed for a writable database: dtor_called();
     if (lock) lock.release();
 }
 
@@ -179,7 +179,7 @@ FlintDatabase::create_and_open_tables(unsigned int block_size)
 
     // Create postlist_table first, and record_table last.  Existence of
     // record_table is considered to imply existence of the database.
-    metafile.create();
+    version_file.create();
     postlist_table.create(block_size);
     positionlist_table.create(block_size);
     termlist_table.create(block_size);
@@ -188,7 +188,6 @@ FlintDatabase::create_and_open_tables(unsigned int block_size)
 
     Assert(database_exists());
 
-    metafile.open();
     record_table.open();
     value_table.open();
     termlist_table.open();
@@ -218,7 +217,7 @@ FlintDatabase::open_tables_consistent()
     // go back and open record_table again, until record_table has
     // the same revision as the last time we opened it.
 
-    metafile.open();
+    version_file.read_and_check();
     record_table.open();
     flint_revision_number_t revision = record_table.get_open_revision_number();
 
@@ -265,7 +264,7 @@ void
 FlintDatabase::open_tables(flint_revision_number_t revision)
 {
     DEBUGCALL(DB, void, "FlintDatabase::open_tables", revision);
-    metafile.open();
+    version_file.read_and_check();
     record_table.open(revision);
     value_table.open(revision);
     termlist_table.open(revision);
@@ -547,6 +546,8 @@ FlintWritableDatabase::~FlintWritableDatabase()
 void
 FlintWritableDatabase::flush()
 {
+    if (transaction_active())
+	throw Xapian::InvalidOperationError("Can't flush during a transaction");
     if (changes_made) do_flush_const();
 }
 
@@ -572,17 +573,16 @@ FlintWritableDatabase::add_document(const Xapian::Document & document)
 {
     DEBUGCALL(DB, Xapian::docid,
 	      "FlintWritableDatabase::add_document", document);
-    RETURN(add_document_(0, document));
+    // Use the next unused document ID.
+    RETURN(add_document_(++lastdocid, document));
 }
 
 Xapian::docid
 FlintWritableDatabase::add_document_(Xapian::docid did,
-				      const Xapian::Document & document)
+				     const Xapian::Document & document)
 {
+    Assert(did != 0);
     try {
-	// Use the next unused document ID.
-	if (did == 0) did = ++lastdocid;
-
 	// Add the record using that document ID.
 	database_ro.record_table.replace_record(document.get_data(), did);
 
@@ -646,13 +646,7 @@ FlintWritableDatabase::add_document_(Xapian::docid did,
 	// transaction, the modifications so far must be cleared before
 	// returning control to the user - otherwise partial modifications will
 	// persist in memory, and eventually get written to disk.
-	database_ro.cancel();
-	total_length = database_ro.postlist_table.get_total_length();
-	lastdocid = database_ro.get_lastdocid();
-	freq_deltas.clear();
-	doclens.clear();
-	mod_plists.clear();
-	changes_made = 0;
+	cancel();
 	throw;
     }
 
@@ -666,7 +660,8 @@ FlintWritableDatabase::add_document_(Xapian::docid did,
     // cout << "+++ mod_plists.size() " << mod_plists.size() <<
     //     ", doclens.size() " << doclens.size() <<
     //	   ", freq_deltas.size() " << freq_deltas.size() << endl;
-    if (++changes_made >= flush_threshold) flush();
+    if (++changes_made >= flush_threshold && !transaction_active())
+	do_flush_const();
 
     return did;
 }
@@ -678,13 +673,6 @@ FlintWritableDatabase::delete_document(Xapian::docid did)
     Assert(did != 0);
 
     try {
-	if (doclens.find(did) != doclens.end()) {
-	    // This document was added or modified in the batch currently
-	    // being buffered.  This should be unusual, and it's fiddly
-	    // to handle, so we just flush and then handle as normal.
-	    do_flush_const();
-	}
-
 	// Remove the record.
 	database_ro.record_table.delete_record(did);
 
@@ -721,8 +709,15 @@ FlintWritableDatabase::delete_document(Xapian::docid did)
 		map<docid, pair<char, termcount> > m;
 		j = mod_plists.insert(make_pair(tname, m)).first;
 	    }
-	    Assert(j->second.find(did) == j->second.end());
-	    j->second.insert(make_pair(did, make_pair('D', 0u)));
+
+	    map<docid, pair<char, termcount> >::iterator k;
+	    k = j->second.find(did);
+	    if (k == j->second.end()) {
+		j->second.insert(make_pair(did, make_pair('D', 0u)));
+	    } else {
+		// Deleting a document we added/modified since the last flush.
+		k->second = make_pair('D', 0u);
+	    }
 
 	    termlist.next();
 	}
@@ -734,36 +729,30 @@ FlintWritableDatabase::delete_document(Xapian::docid did)
 	// transaction, the modifications so far must be cleared before
 	// returning control to the user - otherwise partial modifications will
 	// persist in memory, and eventually get written to disk.
-	database_ro.cancel();
-	total_length = database_ro.postlist_table.get_total_length();
-	lastdocid = database_ro.get_lastdocid();
-	freq_deltas.clear();
-	doclens.clear();
-	mod_plists.clear();
-	changes_made = 0;
+	cancel();
 	throw;
     }
 
-    if (++changes_made >= flush_threshold) flush();
+    if (++changes_made >= flush_threshold && !transaction_active())
+	do_flush_const();
 }
 
 void
 FlintWritableDatabase::replace_document(Xapian::docid did,
-					 const Xapian::Document & document)
+					const Xapian::Document & document)
 {
     DEBUGCALL(DB, void, "FlintWritableDatabase::replace_document", did << ", " << document);
     Assert(did != 0);
 
     try {
-	if (doclens.find(did) != doclens.end()) {
-	    // This document was added or modified in the batch currently
-	    // being buffered.  This should be unusual, and it's fiddly
-	    // to handle, so we just flush and then handle as normal.
-	    do_flush_const();
+	if (did > lastdocid) {
+	    lastdocid = did;
+	    // If this docid is above the highwatermark, then we can't be
+	    // replacing an existing document.
+	    (void)add_document_(did, document);
+	    return;
 	}
 
-	if (did > lastdocid) lastdocid = did;
-
 	// OK, now add entries to remove the postings in the underlying record.
 	Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
 	FlintTermList termlist(ptrtothis,
@@ -791,8 +780,15 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 		map<docid, pair<char, termcount> > m;
 		j = mod_plists.insert(make_pair(tname, m)).first;
 	    }
-	    Assert(j->second.find(did) == j->second.end());
-	    j->second.insert(make_pair(did, make_pair('D', 0u)));
+
+	    map<docid, pair<char, termcount> >::iterator k;
+	    k = j->second.find(did);
+	    if (k == j->second.end()) {
+		j->second.insert(make_pair(did, make_pair('D', 0u)));
+	    } else {
+		// Modifying a document we added/modified since the last flush.
+		k->second = make_pair('D', 0u);
+	    }
 
 	    termlist.next();
 	}
@@ -882,22 +878,18 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 	total_length += new_doclen;
     } catch (const Xapian::DocNotFoundError &) {
 	(void)add_document_(did, document);
+	return;
     } catch (...) {
 	// If an error occurs while replacing a document, or doing any other
 	// transaction, the modifications so far must be cleared before
 	// returning control to the user - otherwise partial modifications will
 	// persist in memory, and eventually get written to disk.
-	database_ro.cancel();
-	total_length = database_ro.postlist_table.get_total_length();
-	lastdocid = database_ro.get_lastdocid();
-	freq_deltas.clear();
-	doclens.clear();
-	mod_plists.clear();
-	changes_made = 0;
+	cancel();
 	throw;
     }
 
-    if (++changes_made >= flush_threshold) flush();
+    if (++changes_made >= flush_threshold && !transaction_active())
+	do_flush_const();
 }
 
 Xapian::doccount
@@ -977,31 +969,35 @@ FlintWritableDatabase::do_open_post_list(const string& tname) const
     DEBUGCALL(DB, LeafPostList *, "FlintWritableDatabase::do_open_post_list", tname);
     Assert(!tname.empty());
 
-    // Need to flush iff we've got buffered changes to this term's postlist.
+    Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
+
     map<string, map<docid, pair<char, termcount> > >::const_iterator j;
     j = mod_plists.find(tname);
-    if (j != mod_plists.end()) do_flush_const();
+    if (j != mod_plists.end()) {
+	// We've got buffered changes to this term's postlist, so we need to
+	// use a FlintModifiedPostList.
+	RETURN(new FlintModifiedPostList(ptrtothis,
+				 &database_ro.postlist_table,
+				 &database_ro.positionlist_table,
+				 tname,
+				 j->second));
+    }
 
-    Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
-    return(new FlintPostList(ptrtothis,
-			      &database_ro.postlist_table,
-			      &database_ro.positionlist_table,
-			      tname));
+    RETURN(new FlintPostList(ptrtothis,
+			     &database_ro.postlist_table,
+			     &database_ro.positionlist_table,
+			     tname));
 }
 
 LeafTermList *
 FlintWritableDatabase::open_term_list(Xapian::docid did) const
 {
-    DEBUGCALL(DB, LeafTermList *, "FlintWritableDatabase::open_term_list",
-	      did);
+    DEBUGCALL(DB, LeafTermList *, "FlintWritableDatabase::open_term_list", did);
     Assert(did != 0);
 
-    // Only need to flush if document #did has been modified.
-    if (doclens.find(did) != doclens.end()) do_flush_const();
-
     Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
     RETURN(new FlintTermList(ptrtothis, &database_ro.termlist_table, did,
-			      get_doccount()));
+			     get_doccount()));
 }
 
 Xapian::Document::Internal *
@@ -1013,9 +1009,9 @@ FlintWritableDatabase::open_document(Xapian::docid did, bool lazy) const
 
     Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
     RETURN(new FlintDocument(ptrtothis,
-			      &database_ro.value_table,
-			      &database_ro.record_table,
-			      did, lazy));
+			     &database_ro.value_table,
+			     &database_ro.record_table,
+			     did, lazy));
 }
 
 PositionList *
@@ -1042,8 +1038,23 @@ TermList *
 FlintWritableDatabase::open_allterms() const
 {
     DEBUGCALL(DB, TermList *, "FlintWritableDatabase::open_allterms", "");
-    // Terms may have been added or removed, so we need to flush.
-    do_flush_const();
+    if (transaction_active())
+	throw Xapian::UnimplementedError("Can't open allterms iterator during a transaction");
+    // If there are changes, terms may have been added or removed, and so we
+    // need to flush.
+    if (changes_made) do_flush_const();
     RETURN(new FlintAllTermsList(Xapian::Internal::RefCntPtr<const FlintWritableDatabase>(this),
 				 &database_ro.postlist_table));
 }
+
+void
+FlintWritableDatabase::cancel()
+{
+    database_ro.cancel();
+    total_length = database_ro.postlist_table.get_total_length();
+    lastdocid = database_ro.get_lastdocid();
+    freq_deltas.clear();
+    doclens.clear();
+    mod_plists.clear();
+    changes_made = 0;
+}
