@@ -57,8 +57,6 @@ PWRITE_PROTOTYPE
 #include <string.h>   /* for memmove */
 #include <limits.h>   /* for CHAR_BIT */
 
-#include "autoptr.h"
-
 #include "btree.h"
 #include "btree_util.h"
 #include "btree_base.h"
@@ -72,6 +70,7 @@ PWRITE_PROTOTYPE
 
 #include <algorithm>  // for std::min()
 #include <string>
+#include <vector>
 
 #ifdef __WIN32__
 # include <io.h> // for _commit()
@@ -91,6 +90,7 @@ PWRITE_PROTOTYPE
 
 using std::min;
 using std::string;
+using std::vector;
 
 //#define BTREE_DEBUG_FULL 1
 #undef BTREE_DEBUG_FULL
