@@ -690,6 +690,8 @@ QuartzWritableDatabase::~QuartzWritableDatabase()
 void
 QuartzWritableDatabase::flush()
 {
+    if (transaction_active())
+	throw Xapian::InvalidOperationError("Can't flush during a transaction");
     if (changes_made) do_flush_const();
 }
 
@@ -715,17 +717,16 @@ QuartzWritableDatabase::add_document(const Xapian::Document & document)
 {
     DEBUGCALL(DB, Xapian::docid,
 	      "QuartzWritableDatabase::add_document", document);
-    RETURN(add_document_(0, document));
+    // Use the next unused document ID.
+    RETURN(add_document_(++lastdocid, document));
 }
 
 Xapian::docid
 QuartzWritableDatabase::add_document_(Xapian::docid did,
 				      const Xapian::Document & document)
 {
+    Assert(did != 0);
     try {
-	// Use the next unused document ID.
-	if (did == 0) did = ++lastdocid;
-
 	// Add the record using that document ID.
 	database_ro.record_table.replace_record(document.get_data(), did);
 
@@ -789,13 +790,7 @@ QuartzWritableDatabase::add_document_(Xapian::docid did,
 	// transaction, the modifications so far must be cleared before
 	// returning control to the user - otherwise partial modifications will
 	// persist in memory, and eventually get written to disk.
-	database_ro.cancel();
-	total_length = database_ro.record_table.get_total_length();
-	lastdocid = database_ro.get_lastdocid();
-	freq_deltas.clear();
-	doclens.clear();
-	mod_plists.clear();
-	changes_made = 0;
+	cancel();
 	throw;
     }
 
@@ -809,7 +804,8 @@ QuartzWritableDatabase::add_document_(Xapian::docid did,
     // cout << "+++ mod_plists.size() " << mod_plists.size() <<
     //     ", doclens.size() " << doclens.size() <<
     //	   ", freq_deltas.size() " << freq_deltas.size() << endl;
-    if (++changes_made >= flush_threshold) flush();
+    if (++changes_made >= flush_threshold && !transaction_active())
+	do_flush_const();
 
     return did;
 }
@@ -821,13 +817,6 @@ QuartzWritableDatabase::delete_document(Xapian::docid did)
     Assert(did != 0);
 
     try {
-	if (doclens.find(did) != doclens.end()) {
-	    // This document was added or modified in the batch currently
-	    // being buffered.  This should be unusual, and it's fiddly
-	    // to handle, so we just flush and then handle as normal.
-	    do_flush_const();
-	}
-
 	// Remove the record.
 	database_ro.record_table.delete_record(did);
 
@@ -864,8 +853,15 @@ QuartzWritableDatabase::delete_document(Xapian::docid did)
 		map<docid, pair<char, termcount> > m;
 		j = mod_plists.insert(make_pair(tname, m)).first;
 	    }
-	    Assert(j->second.find(did) == j->second.end());
-	    j->second.insert(make_pair(did, make_pair('D', 0u)));
+
+	    map<docid, pair<char, termcount> >::iterator k;
+	    k = j->second.find(did);
+	    if (k == j->second.end()) {
+		j->second.insert(make_pair(did, make_pair('D', 0u)));
+	    } else {
+		// Deleting a document we added/modified since the last flush.
+		k->second = make_pair('D', 0u);
+	    }
 
 	    termlist.next();
 	}
@@ -877,17 +873,12 @@ QuartzWritableDatabase::delete_document(Xapian::docid did)
 	// transaction, the modifications so far must be cleared before
 	// returning control to the user - otherwise partial modifications will
 	// persist in memory, and eventually get written to disk.
-	database_ro.cancel();
-	total_length = database_ro.record_table.get_total_length();
-	lastdocid = database_ro.get_lastdocid();
-	freq_deltas.clear();
-	doclens.clear();
-	mod_plists.clear();
-	changes_made = 0;
+	cancel();
 	throw;
     }
 
-    if (++changes_made >= flush_threshold) flush();
+    if (++changes_made >= flush_threshold && !transaction_active())
+	do_flush_const();
 }
 
 void
@@ -898,15 +889,14 @@ QuartzWritableDatabase::replace_document(Xapian::docid did,
     Assert(did != 0);
 
     try {
-	if (doclens.find(did) != doclens.end()) {
-	    // This document was added or modified in the batch currently
-	    // being buffered.  This should be unusual, and it's fiddly
-	    // to handle, so we just flush and then handle as normal.
-	    do_flush_const();
+	if (did > lastdocid) {
+	    lastdocid = did;
+	    // If this docid is above the highwatermark, then we can't be
+	    // replacing an existing document.
+	    (void)add_document_(did, document);
+	    return;
 	}
 
-	if (did > lastdocid) lastdocid = did;
-
 	// OK, now add entries to remove the postings in the underlying record.
 	Xapian::Internal::RefCntPtr<const QuartzWritableDatabase> ptrtothis(this);
 	QuartzTermList termlist(ptrtothis,
@@ -934,8 +924,15 @@ QuartzWritableDatabase::replace_document(Xapian::docid did,
 		map<docid, pair<char, termcount> > m;
 		j = mod_plists.insert(make_pair(tname, m)).first;
 	    }
-	    Assert(j->second.find(did) == j->second.end());
-	    j->second.insert(make_pair(did, make_pair('D', 0u)));
+
+	    map<docid, pair<char, termcount> >::iterator k;
+	    k = j->second.find(did);
+	    if (k == j->second.end()) {
+		j->second.insert(make_pair(did, make_pair('D', 0u)));
+	    } else {
+		// Modifying a document we added/modified since the last flush.
+		k->second = make_pair('D', 0u);
+	    }
 
 	    termlist.next();
 	}
@@ -1025,22 +1022,18 @@ QuartzWritableDatabase::replace_document(Xapian::docid did,
 	total_length += new_doclen;
     } catch (const Xapian::DocNotFoundError &) {
 	(void)add_document_(did, document);
+	return;
     } catch (...) {
 	// If an error occurs while replacing a document, or doing any other
 	// transaction, the modifications so far must be cleared before
 	// returning control to the user - otherwise partial modifications will
 	// persist in memory, and eventually get written to disk.
-	database_ro.cancel();
-	total_length = database_ro.record_table.get_total_length();
-	lastdocid = database_ro.get_lastdocid();
-	freq_deltas.clear();
-	doclens.clear();
-	mod_plists.clear();
-	changes_made = 0;
+	cancel();
 	throw;
     }
 
-    if (++changes_made >= flush_threshold) flush();
+    if (++changes_made >= flush_threshold && !transaction_active())
+	do_flush_const();
 }
 
 Xapian::doccount
@@ -1123,7 +1116,11 @@ QuartzWritableDatabase::do_open_post_list(const string& tname) const
     // Need to flush iff we've got buffered changes to this term's postlist.
     map<string, map<docid, pair<char, termcount> > >::const_iterator j;
     j = mod_plists.find(tname);
-    if (j != mod_plists.end()) do_flush_const();
+    if (j != mod_plists.end()) {
+	if (transaction_active())
+	    throw Xapian::UnimplementedError("Can't open modified postlist during a transaction");
+	do_flush_const();
+    }
 
     Xapian::Internal::RefCntPtr<const QuartzWritableDatabase> ptrtothis(this);
     return(new QuartzPostList(ptrtothis,
@@ -1139,9 +1136,6 @@ QuartzWritableDatabase::open_term_list(Xapian::docid did) const
 	      did);
     Assert(did != 0);
 
-    // Only need to flush if document #did has been modified.
-    if (doclens.find(did) != doclens.end()) do_flush_const();
-
     Xapian::Internal::RefCntPtr<const QuartzWritableDatabase> ptrtothis(this);
     RETURN(new QuartzTermList(ptrtothis, &database_ro.termlist_table, did,
 			      get_doccount()));
@@ -1185,10 +1179,24 @@ TermList *
 QuartzWritableDatabase::open_allterms() const
 {
     DEBUGCALL(DB, TermList *, "QuartzWritableDatabase::open_allterms", "");
+    if (transaction_active())
+	throw Xapian::UnimplementedError("Can't open allterms iterator during a transaction");
     // Terms may have been added or removed, so we need to flush.
-    do_flush_const();
+    if (changes_made) do_flush_const();
     QuartzPostListTable *t = &database_ro.postlist_table;
     AutoPtr<Bcursor> pl_cursor(t->cursor_get());
     RETURN(new QuartzAllTermsList(Xapian::Internal::RefCntPtr<const QuartzWritableDatabase>(this),
 				  pl_cursor, t->get_entry_count()));
 }
+
+void
+QuartzWritableDatabase::cancel()
+{
+    database_ro.cancel();
+    total_length = database_ro.record_table.get_total_length();
+    lastdocid = database_ro.get_lastdocid();
+    freq_deltas.clear();
+    doclens.clear();
+    mod_plists.clear();
+    changes_made = 0;
+}
