@@ -134,9 +134,15 @@ main(int argc, char **argv)
 	    exit(1);
 	}
 	cout << "No errors found" << endl;
+    } catch (const char *error) {
+	cerr << argv[0] << ": " << error << endl;
+	exit(1);
     } catch (const Xapian::Error &error) {
 	cerr << argv[0] << ": " << error.get_msg() << endl;
 	exit(1);
+    } catch (...) {
+	cerr << argv[0] << ": Unknown exception" << endl;
+	exit(1);
     }
 }
 
@@ -526,8 +532,6 @@ check_table(const char *filename, int opts)
 	    pos = cursor->current_tag.data();
 	    end = pos + cursor->current_tag.size();
 
-	    bool bad = false;
-
 	    bool first = true;
 	    Xapian::valueno last_value_no = 0;
 	    while (pos && pos != end) {
@@ -540,7 +544,6 @@ check_table(const char *filename, int opts)
 		    else
 			cout << "Value number in value table is too large" << endl;
 		    ++errors;
-		    bad = true;
 		    break;
 		}
 
@@ -550,7 +553,6 @@ check_table(const char *filename, int opts)
 		    else
 			cout << "Item in value table is too large" << endl;
 		    ++errors;
-		    bad = true;
 		    break;
 		}
 
