@@ -22,17 +22,15 @@
 #ifndef OM_HGUARD_MULTIMATCH_H
 #define OM_HGUARD_MULTIMATCH_H
 
-#include "match.h"
 #include "stats.h"
 
 #include "omqueryinternal.h"
+#include "submatch.h"
 
 #include <vector>
 #include <map>
 #include "autoptr.h"  // auto_ptr
 
-class SubMatch;
-
 class MultiMatch
 {
     private:
@@ -75,13 +73,6 @@ class MultiMatch
 	 */
         bool recalculate_w_max;
 
-	/** Prepare all the sub matchers.
-	 *
-	 *  This calls prepare_match() on all the sub matchers until they
-	 *  all return true to signal that they are prepared.
-	 */
-	void prepare_matchers();
-
 	/// get the collapse key
 	string get_collapse_key(PostList *pl,
 				Xapian::docid did, Xapian::valueno keyno,
@@ -128,7 +119,6 @@ class MultiMatch
 		   Xapian::ErrorHandler * errorhandler,
 		   StatsGatherer * gatherer_,
 		   const Xapian::Weight *wtscheme);
-	~MultiMatch();
 
 	void get_mset(Xapian::doccount first,
 		      Xapian::doccount maxitems,
