@@ -50,6 +50,10 @@ class Document;
  */
 class Database {
     public:
+	class Internal;
+	/// @internal Reference counted internals.
+	std::vector<Xapian::Internal::RefCntPtr<Internal> > internal;
+
 	/** Add an existing database (or group of databases) to those
 	 *  accessed by this object.
 	 *
@@ -57,12 +61,6 @@ class Database {
 	 */
 	void add_database(const Database & database);
 
-    public:
-	class Internal;
-	/// @internal Reference counted internals.
-	std::vector<Xapian::Internal::RefCntPtr<Internal> > internal;
-
-    public:
 	/** Create a Database with no databases in.
 	 */
 	Database();
@@ -130,6 +128,9 @@ class Database {
 	    return TermIterator(NULL);
 	}
 
+	/** Does this database have any positional information? */
+	bool has_positions() const;
+
 	/** An iterator pointing to the start of the position list
 	 *  for a given term in a given document.
 	 */
@@ -214,7 +215,7 @@ class WritableDatabase : public Database {
 	 *
 	 *  If there are no copies of this object remaining, the database
 	 *  will be closed.  If there are any transactions in progress
-	 *  these will be ended.
+	 *  these will be aborted as if cancel_transaction had been called.
 	 */
 	virtual ~WritableDatabase();
 
@@ -276,10 +277,7 @@ class WritableDatabase : public Database {
 	 *  performed or not performed at all: it is then up to the
 	 *  application to work out which operations need to be repeated.
 	 *
-	 *  If called within a transaction, this will flush database
-	 *  modifications made before the transaction was begun, but will
-	 *  not flush modifications made since begin_transaction() was
-	 *  called.
+	 *  It's not valid to call flush within a transaction.
 	 *
 	 *  Beware of calling flush too frequently: this will have a severe
 	 *  performance cost.
@@ -301,15 +299,35 @@ class WritableDatabase : public Database {
 
 	/** Begin a transaction.
 	 *
-	 *  For the purposes of Xapian, a transaction is a group of
-	 *  modifications to the database which are grouped together such
-	 *  that either all or none of them will succeed.  Even in the case
-	 *  of a power failure, this characteristic should be preserved (as
-	 *  long as the filesystem isn't corrupted, etc).
+	 *  In Xapian a transaction is a group of modifications to the database
+	 *  which are linked such that either all will be applied
+	 *  simultaneously or none will be applied at all.  Even in the case of
+	 *  a power failure, this characteristic should be preserved (as long
+	 *  as the filesystem isn't corrupted, etc).
 	 *
-	 *  Transactions are only available with certain access methods,
-	 *  and as you might expect will generally have a fairly high
-	 *  performance cost.
+	 *  A transaction is started with begin_transaction() and can
+	 *  either be committed by calling commit_transaction() or aborted
+	 *  by calling cancel_transaction().
+	 *
+	 *  By default, a transaction implicitly calls flush before and after
+	 *  so that the modifications stand and fall without affecting
+	 *  modifications before or after.
+	 *
+	 *  The downside of this flushing is that small transactions cause
+	 *  modifications to be frequently flushed which can harm indexing
+	 *  performance in the same way that explicitly calling flush
+	 *  frequently can.
+	 *
+	 *  If you're applying atomic groups of changes and only wish to
+	 *  ensure that each group is either applied or not applied, then
+	 *  you can prevent the automatic flush before and after the
+	 *  transaction by starting the transaction with
+	 *  begin_transaction(false).  However, if cancel_transaction is
+	 *  called (or if commit_transaction isn't called before the
+	 *  WritableDatabase object is destroyed) then any changes which
+	 *  were pending before the transaction began will also be discarded.
+	 *
+	 *  Transactions aren't currently supported by the InMemory backend.
 	 *
 	 *  @exception Xapian::UnimplementedError will be thrown if transactions
 	 *             are not available for this database type.
@@ -318,12 +336,12 @@ class WritableDatabase : public Database {
 	 *             called at an invalid time, such as when a transaction
 	 *             is already in progress.
 	 */
-	void begin_transaction();
+	void begin_transaction(bool flushed=true);
 
-	/** End the transaction currently in progress, committing the
-	 *  modifications made to the database.
+	/** Complete the transaction currently in progress.
 	 *
-	 *  If this completes successfully, all the database modifications
+	 *  If this method completes successfully and this is a flushed
+	 *  transaction, all the database modifications
 	 *  made during the transaction will have been committed to the
 	 *  database.
 	 *
@@ -331,8 +349,7 @@ class WritableDatabase : public Database {
 	 *  the modifications made to the database during the transaction
 	 *  will have been applied to the database.
 	 *
-	 *  Whatever occurs, after this method the transaction will no
-	 *  longer be in progress.
+	 *  In all cases the transaction will no longer be in progress.
 	 *
 	 *  @exception Xapian::DatabaseError will be thrown if a problem occurs
 	 *             while modifying the database.
@@ -348,7 +365,7 @@ class WritableDatabase : public Database {
 	 */
 	void commit_transaction();
 
-	/** End the transaction currently in progress, cancelling the
+	/** Abort the transaction currently in progress, discarding the
 	 *  potential modifications made to the database.
 	 *
 	 *  If an error occurs in this method, an exception will be thrown,
@@ -371,7 +388,9 @@ class WritableDatabase : public Database {
 	/** Add a new document to the database.
 	 *
 	 *  This method adds the specified document to the database,
-	 *  returning a newly allocated document ID.
+	 *  returning a newly allocated document ID.  Automatically allocated
+	 *  document IDs come from a per-database monotonically increasing
+	 *  counter, so IDs from deleted documents won't be reused.
 	 *
 	 *  Note that this does not mean the document will immediately
 	 *  appear in the database; see flush() for more details.
