@@ -4,14 +4,13 @@
 #define XOR                             4
 #define AND                             5
 #define NEAR                            6
-#define LOVE                            7
-#define HATE                            8
-#define TERM                            9
-#define PHR_TERM                       10
-#define WILD_TERM                      11
-#define BOOLEAN_FILTER                 12
-#define PREFIX_QUOTE                   13
+#define ADJ                             7
+#define LOVE                            8
+#define HATE                            9
+#define TERM                           10
+#define PHR_TERM                       11
+#define WILD_TERM                      12
+#define BOOLEAN_FILTER                 13
 #define QUOTE                          14
 #define BRA                            15
 #define KET                            16
-#define PREFIX_BRA                     17
