@@ -30,7 +30,6 @@
 #include <xapian.h>
 #include "testsuite.h"
 #include "testutils.h"
-#include "backendmanager.h"
 #include "utils.h"
 
 #include "apitest.h"
@@ -40,8 +39,6 @@
 
 using namespace std;
 
-extern BackendManager backendmanager;
-
 static Xapian::Query
 query(const string &t)
 {
@@ -72,7 +69,8 @@ static bool test_stubdb1()
 {
     ofstream out("stubdb1");
     TEST(out.is_open());
-    out << "remote :../bin/xapian-progsrv " << backendmanager.get_datadir() << " apitest_simpledata\n";
+    // FIXME: not very reliable...
+    out << "remote :../bin/xapian-progsrv .flint/db=apitest_simpledata\n";
     out.close();
 
     {
@@ -931,15 +929,7 @@ static bool test_postlist2()
     Xapian::PostingIterator p_clone(p);
     TEST_EQUAL(p, p_clone);
 
-#ifdef __SUNPRO_CC
-    vector<Xapian::docid> v;
-    while (p != pend) {
-	v.push_back(*p);
-	++p;
-    }
-#else
     vector<Xapian::docid> v(p, pend);
-#endif
 
     p = db.postlist_begin("this");
     pend = db.postlist_end("this");
